@@ -16,7 +16,7 @@
  * ----------------------------------------------------------------------------
  *
  * Copyright (c) 2009-2012, Pieter Noordhuis <pcnoordhuis at gmail dot com>
- * Copyright (c) 2009-2012, Salvatore Sanfilippo <antirez at gmail dot com>
+ * Copyright (c) 2009-2012, Redis Ltd.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -55,21 +55,21 @@
 #include "crc64.h"
 #include "config.h"
 #include "server.h"
+#include "connhelpers.h"
 
 /* ------------------------- Buffer I/O implementation ----------------------- */
 
 /* Returns 1 or 0 for success/failure. */
 static size_t rioBufferWrite(rio *r, const void *buf, size_t len) {
-    r->io.buffer.ptr = sdscatlen(r->io.buffer.ptr,(char*)buf,len);
+    r->io.buffer.ptr = sdscatlen(r->io.buffer.ptr, (char *)buf, len);
     r->io.buffer.pos += len;
     return 1;
 }
 
 /* Returns 1 or 0 for success/failure. */
 static size_t rioBufferRead(rio *r, void *buf, size_t len) {
-    if (sdslen(r->io.buffer.ptr)-r->io.buffer.pos < len)
-        return 0; /* not enough buffer to return len bytes. */
-    memcpy(buf,r->io.buffer.ptr+r->io.buffer.pos,len);
+    if (sdslen(r->io.buffer.ptr) - r->io.buffer.pos < len) return 0; /* not enough buffer to return len bytes. */
+    memcpy(buf, r->io.buffer.ptr + r->io.buffer.pos, len);
     r->io.buffer.pos += len;
     return 1;
 }
@@ -91,12 +91,12 @@ static const rio rioBufferIO = {
     rioBufferWrite,
     rioBufferTell,
     rioBufferFlush,
-    NULL,           /* update_checksum */
-    0,              /* current checksum */
-    0,              /* flags */
-    0,              /* bytes read or written */
-    0,              /* read/write chunk size */
-    { { NULL, 0 } } /* union for io-specific vars */
+    NULL,       /* update_checksum */
+    0,          /* current checksum */
+    0,          /* flags */
+    0,          /* bytes read or written */
+    0,          /* read/write chunk size */
+    {{NULL, 0}} /* union for io-specific vars */
 };
 
 void rioInitWithBuffer(rio *r, sds s) {
@@ -109,7 +109,7 @@ void rioInitWithBuffer(rio *r, sds s) {
 
 /* Returns 1 or 0 for success/failure. */
 static size_t rioFileWrite(rio *r, const void *buf, size_t len) {
-    if (!r->io.file.autosync) return fwrite(buf,len,1,r->io.file.fp);
+    if (!r->io.file.autosync) return fwrite(buf, len, 1, r->io.file.fp);
 
     size_t nwritten = 0;
     /* Incrementally write data to the file, avoid a single write larger than
@@ -118,9 +118,9 @@ static size_t rioFileWrite(rio *r, const void *buf, size_t len) {
     while (len != nwritten) {
         serverAssert(r->io.file.autosync > r->io.file.buffered);
         size_t nalign = (size_t)(r->io.file.autosync - r->io.file.buffered);
-        size_t towrite = nalign > len-nwritten ? len-nwritten : nalign;
+        size_t towrite = nalign > len - nwritten ? len - nwritten : nalign;
 
-        if (fwrite((char*)buf+nwritten,towrite,1,r->io.file.fp) == 0) return 0;
+        if (fwrite((char *)buf + nwritten, towrite, 1, r->io.file.fp) == 0) return 0;
         nwritten += towrite;
         r->io.file.buffered += towrite;
 
@@ -133,31 +133,29 @@ static size_t rioFileWrite(rio *r, const void *buf, size_t len) {
 
 #if HAVE_SYNC_FILE_RANGE
             /* Start writeout asynchronously. */
-            if (sync_file_range(fileno(r->io.file.fp),
-                    processed - r->io.file.autosync, r->io.file.autosync,
-                    SYNC_FILE_RANGE_WRITE) == -1)
+            if (sync_file_range(fileno(r->io.file.fp), processed - r->io.file.autosync, r->io.file.autosync,
+                                SYNC_FILE_RANGE_WRITE) == -1)
                 return 0;
 
             if (processed >= (size_t)r->io.file.autosync * 2) {
                 /* To keep the promise to 'autosync', we should make sure last
                  * asynchronous writeout persists into disk. This call may block
                  * if last writeout is not finished since disk is slow. */
-                if (sync_file_range(fileno(r->io.file.fp),
-                        processed - r->io.file.autosync*2,
-                        r->io.file.autosync, SYNC_FILE_RANGE_WAIT_BEFORE|
-                        SYNC_FILE_RANGE_WRITE|SYNC_FILE_RANGE_WAIT_AFTER) == -1)
+                if (sync_file_range(fileno(r->io.file.fp), processed - r->io.file.autosync * 2, r->io.file.autosync,
+                                    SYNC_FILE_RANGE_WAIT_BEFORE | SYNC_FILE_RANGE_WRITE | SYNC_FILE_RANGE_WAIT_AFTER) ==
+                    -1)
                     return 0;
             }
 #else
-            if (redis_fsync(fileno(r->io.file.fp)) == -1) return 0;
+            if (valkey_fsync(fileno(r->io.file.fp)) == -1) return 0;
 #endif
             if (r->io.file.reclaim_cache) {
                 /* In Linux sync_file_range just issue a writeback request to
                  * OS, and when posix_fadvise is called, the dirty page may
                  * still be in flushing, which means it would be ignored by
                  * posix_fadvise.
-                 * 
-                 * So we posix_fadvise the whole file, and the writeback-ed 
+                 *
+                 * So we posix_fadvise the whole file, and the writeback-ed
                  * pages will have other chances to be reclaimed. */
                 reclaimFilePageCache(fileno(r->io.file.fp), 0, 0);
             }
@@ -169,7 +167,7 @@ static size_t rioFileWrite(rio *r, const void *buf, size_t len) {
 
 /* Returns 1 or 0 for success/failure. */
 static size_t rioFileRead(rio *r, void *buf, size_t len) {
-    return fread(buf,len,1,r->io.file.fp);
+    return fread(buf, len, 1, r->io.file.fp);
 }
 
 /* Returns read/write position in file. */
@@ -184,16 +182,12 @@ static int rioFileFlush(rio *r) {
 }
 
 static const rio rioFileIO = {
-    rioFileRead,
-    rioFileWrite,
-    rioFileTell,
-    rioFileFlush,
-    NULL,           /* update_checksum */
-    0,              /* current checksum */
-    0,              /* flags */
-    0,              /* bytes read or written */
-    0,              /* read/write chunk size */
-    { { NULL, 0 } } /* union for io-specific vars */
+    rioFileRead, rioFileWrite, rioFileTell, rioFileFlush, NULL, /* update_checksum */
+    0,                                                          /* current checksum */
+    0,                                                          /* flags */
+    0,                                                          /* bytes read or written */
+    0,                                                          /* read/write chunk size */
+    {{NULL, 0}}                                                 /* union for io-specific vars */
 };
 
 void rioInitWithFile(rio *r, FILE *fp) {
@@ -219,7 +213,7 @@ static size_t rioConnWrite(rio *r, const void *buf, size_t len) {
 
 /* Returns 1 or 0 for success/failure. */
 static size_t rioConnRead(rio *r, void *buf, size_t len) {
-    size_t avail = sdslen(r->io.conn.buf)-r->io.conn.pos;
+    size_t avail = sdslen(r->io.conn.buf) - r->io.conn.pos;
 
     /* If the buffer is too small for the entire request: realloc. */
     if (sdslen(r->io.conn.buf) + sdsavail(r->io.conn.buf) < len)
@@ -246,16 +240,12 @@ static size_t rioConnRead(rio *r, void *buf, size_t len) {
         size_t needs = len - buffered;
         /* Read either what's missing, or PROTO_IOBUF_LEN, the bigger of
          * the two. */
-        size_t toread = needs < PROTO_IOBUF_LEN ? PROTO_IOBUF_LEN: needs;
+        size_t toread = needs < PROTO_IOBUF_LEN ? PROTO_IOBUF_LEN : needs;
         if (toread > sdsavail(r->io.conn.buf)) toread = sdsavail(r->io.conn.buf);
-        if (r->io.conn.read_limit != 0 &&
-            r->io.conn.read_so_far + buffered + toread > r->io.conn.read_limit)
-        {
+        if (r->io.conn.read_limit != 0 && r->io.conn.read_so_far + buffered + toread > r->io.conn.read_limit) {
             toread = r->io.conn.read_limit - r->io.conn.read_so_far - buffered;
         }
-        int retval = connRead(r->io.conn.conn,
-                          (char*)r->io.conn.buf + sdslen(r->io.conn.buf),
-                          toread);
+        int retval = connRead(r->io.conn.conn, (char *)r->io.conn.buf + sdslen(r->io.conn.buf), toread);
         if (retval == 0) {
             return 0;
         } else if (retval < 0) {
@@ -266,7 +256,7 @@ static size_t rioConnRead(rio *r, void *buf, size_t len) {
         sdsIncrLen(r->io.conn.buf, retval);
     }
 
-    memcpy(buf, (char*)r->io.conn.buf + r->io.conn.pos, len);
+    memcpy(buf, (char *)r->io.conn.buf + r->io.conn.pos, len);
     r->io.conn.read_so_far += len;
     r->io.conn.pos += len;
     return len;
@@ -282,20 +272,16 @@ static off_t rioConnTell(rio *r) {
 static int rioConnFlush(rio *r) {
     /* Our flush is implemented by the write method, that recognizes a
      * buffer set to NULL with a count of zero as a flush request. */
-    return rioConnWrite(r,NULL,0);
+    return rioConnWrite(r, NULL, 0);
 }
 
 static const rio rioConnIO = {
-    rioConnRead,
-    rioConnWrite,
-    rioConnTell,
-    rioConnFlush,
-    NULL,           /* update_checksum */
-    0,              /* current checksum */
-    0,              /* flags */
-    0,              /* bytes read or written */
-    0,              /* read/write chunk size */
-    { { NULL, 0 } } /* union for io-specific vars */
+    rioConnRead, rioConnWrite, rioConnTell, rioConnFlush, NULL, /* update_checksum */
+    0,                                                          /* current checksum */
+    0,                                                          /* flags */
+    0,                                                          /* bytes read or written */
+    0,                                                          /* read/write chunk size */
+    {{NULL, 0}}                                                 /* union for io-specific vars */
 };
 
 /* Create an RIO that implements a buffered read from an fd
@@ -324,7 +310,7 @@ void rioFreeConn(rio *r, sds *remaining) {
 }
 
 /* ------------------- File descriptor implementation ------------------
- * This target is used to write the RDB file to pipe, when the master just
+ * This target is used to write the RDB file to pipe, when the primary just
  * streams the data to the replicas without creating an RDB on-disk image
  * (diskless replication option).
  * It only implements writes. */
@@ -336,7 +322,7 @@ void rioFreeConn(rio *r, sds *remaining) {
  * to implement rioFdFlush(). */
 static size_t rioFdWrite(rio *r, const void *buf, size_t len) {
     ssize_t retval;
-    unsigned char *p = (unsigned char*) buf;
+    unsigned char *p = (unsigned char *)buf;
     int doflush = (buf == NULL && len == 0);
 
     /* For small writes, we rather keep the data in user-space buffer, and flush
@@ -346,26 +332,23 @@ static size_t rioFdWrite(rio *r, const void *buf, size_t len) {
     if (len > PROTO_IOBUF_LEN) {
         /* First, flush any pre-existing buffered data. */
         if (sdslen(r->io.fd.buf)) {
-            if (rioFdWrite(r, NULL, 0) == 0)
-                return 0;
+            if (rioFdWrite(r, NULL, 0) == 0) return 0;
         }
         /* Write the new data, keeping 'p' and 'len' from the input. */
     } else {
         if (len) {
-            r->io.fd.buf = sdscatlen(r->io.fd.buf,buf,len);
-            if (sdslen(r->io.fd.buf) > PROTO_IOBUF_LEN)
-                doflush = 1;
-            if (!doflush)
-                return 1;
+            r->io.fd.buf = sdscatlen(r->io.fd.buf, buf, len);
+            if (sdslen(r->io.fd.buf) > PROTO_IOBUF_LEN) doflush = 1;
+            if (!doflush) return 1;
         }
         /* Flushing the buffered data. set 'p' and 'len' accordingly. */
-        p = (unsigned char*) r->io.fd.buf;
+        p = (unsigned char *)r->io.fd.buf;
         len = sdslen(r->io.fd.buf);
     }
 
     size_t nwritten = 0;
-    while(nwritten != len) {
-        retval = write(r->io.fd.fd,p+nwritten,len-nwritten);
+    while (nwritten != len) {
+        retval = write(r->io.fd.fd, p + nwritten, len - nwritten);
         if (retval <= 0) {
             if (retval == -1 && errno == EINTR) continue;
             /* With blocking io, which is the sole user of this
@@ -401,20 +384,16 @@ static off_t rioFdTell(rio *r) {
 static int rioFdFlush(rio *r) {
     /* Our flush is implemented by the write method, that recognizes a
      * buffer set to NULL with a count of zero as a flush request. */
-    return rioFdWrite(r,NULL,0);
+    return rioFdWrite(r, NULL, 0);
 }
 
 static const rio rioFdIO = {
-    rioFdRead,
-    rioFdWrite,
-    rioFdTell,
-    rioFdFlush,
-    NULL,           /* update_checksum */
-    0,              /* current checksum */
-    0,              /* flags */
-    0,              /* bytes read or written */
-    0,              /* read/write chunk size */
-    { { NULL, 0 } } /* union for io-specific vars */
+    rioFdRead,  rioFdWrite, rioFdTell, rioFdFlush, NULL, /* update_checksum */
+    0,                                                   /* current checksum */
+    0,                                                   /* flags */
+    0,                                                   /* bytes read or written */
+    0,                                                   /* read/write chunk size */
+    {{NULL, 0}}                                          /* union for io-specific vars */
 };
 
 void rioInitWithFd(rio *r, int fd) {
@@ -434,7 +413,7 @@ void rioFreeFd(rio *r) {
 /* This function can be installed both in memory and file streams when checksum
  * computation is needed. */
 void rioGenericUpdateChecksum(rio *r, const void *buf, size_t len) {
-    r->cksum = crc64(r->cksum,buf,len);
+    r->cksum = crc64(r->cksum, buf, len);
 }
 
 /* Set the file-based rio object to auto-fsync every 'bytes' file written.
@@ -446,14 +425,14 @@ void rioGenericUpdateChecksum(rio *r, const void *buf, size_t len) {
  * disk I/O concentrated in very little time. When we fsync in an explicit
  * way instead the I/O pressure is more distributed across time. */
 void rioSetAutoSync(rio *r, off_t bytes) {
-    if(r->write != rioFileIO.write) return;
+    if (r->write != rioFileIO.write) return;
     r->io.file.autosync = bytes;
 }
 
 /* Set the file-based rio object to reclaim cache after every auto-sync.
  * In the Linux implementation POSIX_FADV_DONTNEED skips the dirty
  * pages, so if auto sync is unset this option will have no effect.
- * 
+ *
  * This feature can reduce the cache footprint backed by the file. */
 void rioSetReclaimCache(rio *r, int enabled) {
     r->io.file.reclaim_cache = enabled;
@@ -476,7 +455,7 @@ uint8_t rioCheckType(rio *r) {
 /* --------------------------- Higher level interface --------------------------
  *
  * The following higher level functions use lower level rio.c functions to help
- * generating the Redis protocol for the Append Only File. */
+ * generating the RESP for the Append Only File. */
 
 /* Write multi bulk count in the format: "*<count>\r\n". */
 size_t rioWriteBulkCount(rio *r, char prefix, long count) {
@@ -484,10 +463,10 @@ size_t rioWriteBulkCount(rio *r, char prefix, long count) {
     int clen;
 
     cbuf[0] = prefix;
-    clen = 1+ll2string(cbuf+1,sizeof(cbuf)-1,count);
+    clen = 1 + ll2string(cbuf + 1, sizeof(cbuf) - 1, count);
     cbuf[clen++] = '\r';
     cbuf[clen++] = '\n';
-    if (rioWrite(r,cbuf,clen) == 0) return 0;
+    if (rioWrite(r, cbuf, clen) == 0) return 0;
     return clen;
 }
 
@@ -495,10 +474,10 @@ size_t rioWriteBulkCount(rio *r, char prefix, long count) {
 size_t rioWriteBulkString(rio *r, const char *buf, size_t len) {
     size_t nwritten;
 
-    if ((nwritten = rioWriteBulkCount(r,'$',len)) == 0) return 0;
-    if (len > 0 && rioWrite(r,buf,len) == 0) return 0;
-    if (rioWrite(r,"\r\n",2) == 0) return 0;
-    return nwritten+len+2;
+    if ((nwritten = rioWriteBulkCount(r, '$', len)) == 0) return 0;
+    if (len > 0 && rioWrite(r, buf, len) == 0) return 0;
+    if (rioWrite(r, "\r\n", 2) == 0) return 0;
+    return nwritten + len + 2;
 }
 
 /* Write a long long value in format: "$<count>\r\n<payload>\r\n". */
@@ -506,8 +485,8 @@ size_t rioWriteBulkLongLong(rio *r, long long l) {
     char lbuf[32];
     unsigned int llen;
 
-    llen = ll2string(lbuf,sizeof(lbuf),l);
-    return rioWriteBulkString(r,lbuf,llen);
+    llen = ll2string(lbuf, sizeof(lbuf), l);
+    return rioWriteBulkString(r, lbuf, llen);
 }
 
 /* Write a double value in the format: "$<count>\r\n<payload>\r\n" */
@@ -516,5 +495,130 @@ size_t rioWriteBulkDouble(rio *r, double d) {
     unsigned int dlen;
     dlen = fpconv_dtoa(d, dbuf);
     dbuf[dlen] = '\0';
-    return rioWriteBulkString(r,dbuf,dlen);
+    return rioWriteBulkString(r, dbuf, dlen);
+}
+
+/* Returns 1 or 0 for success/failure.
+ * The function returns success as long as we are able to correctly write
+ * to at least one file descriptor.
+ *
+ * When buf is NULL and len is 0, the function performs a flush operation
+ * if there is some pending buffer, so this function is also used in order
+ * to implement rioConnsetFlush(). */
+static size_t rioConnsetWrite(rio *r, const void *buf, size_t len) {
+    ssize_t retval;
+    int j;
+    unsigned char *p = (unsigned char *)buf;
+    int doflush = (buf == NULL && len == 0);
+
+    /* To start we always append to our buffer. If it gets larger than
+     * a given size, we actually write to the sockets. */
+    if (len) {
+        r->io.connset.buf = sdscatlen(r->io.connset.buf, buf, len);
+        len = 0; /* Prevent entering the while below if we don't flush. */
+        if (sdslen(r->io.connset.buf) > PROTO_IOBUF_LEN) doflush = 1;
+    }
+
+    if (doflush) {
+        p = (unsigned char *)r->io.connset.buf;
+        len = sdslen(r->io.connset.buf);
+    }
+
+    /* Write in little chunchs so that when there are big writes we
+     * parallelize while the kernel is sending data in background to
+     * the TCP socket. */
+    while (len) {
+        size_t count = len < RIO_CONNSET_WRITE_MAX_CHUNK_SIZE ? len : RIO_CONNSET_WRITE_MAX_CHUNK_SIZE;
+        int broken = 0;
+        for (j = 0; j < r->io.connset.numconns; j++) {
+            if (r->io.connset.state[j] != 0) {
+                /* Skip FDs already in error. */
+                broken++;
+                continue;
+            }
+
+            /* Make sure to write 'count' bytes to the socket regardless
+             * of short writes. */
+            size_t nwritten = 0;
+            while (nwritten != count) {
+                retval = connWrite(r->io.connset.conns[j], p + nwritten, count - nwritten);
+                if (retval <= 0) {
+                    /* With blocking sockets, which is the sole user of this
+                     * rio target, EWOULDBLOCK is returned only because of
+                     * the SO_SNDTIMEO socket option, so we translate the error
+                     * into one more recognizable by the user. */
+                    if (retval == -1 && errno == EWOULDBLOCK) errno = ETIMEDOUT;
+                    break;
+                }
+                nwritten += retval;
+            }
+
+            if (nwritten != count) {
+                /* Mark this FD as broken. */
+                r->io.connset.state[j] = errno;
+                if (r->io.connset.state[j] == 0) r->io.connset.state[j] = EIO;
+            }
+        }
+        if (broken == r->io.connset.numconns) return 0; /* All the FDs in error. */
+        p += count;
+        len -= count;
+        r->io.connset.pos += count;
+    }
+
+    if (doflush) sdsclear(r->io.connset.buf);
+    return 1;
+}
+
+/* Returns 1 or 0 for success/failure. */
+static size_t rioConnsetRead(rio *r, void *buf, size_t len) {
+    UNUSED(r);
+    UNUSED(buf);
+    UNUSED(len);
+    return 0; /* Error, this target does not support reading. */
+}
+
+/* Returns read/write position in file. */
+static off_t rioConnsetTell(rio *r) {
+    return r->io.connset.pos;
+}
+
+/* Flushes any buffer to target device if applicable. Returns 1 on success
+ * and 0 on failures. */
+static int rioConnsetFlush(rio *r) {
+    /* Our flush is implemented by the write method, that recognizes a
+     * buffer set to NULL with a count of zero as a flush request. */
+    return rioConnsetWrite(r, NULL, 0);
+}
+
+static const rio rioConnsetIO = {
+    rioConnsetRead,
+    rioConnsetWrite,
+    rioConnsetTell,
+    rioConnsetFlush,
+    NULL,       /* update_checksum */
+    0,          /* current checksum */
+    0,          /* flags */
+    0,          /* bytes read or written */
+    0,          /* read/write chunk size */
+    {{NULL, 0}} /* union for io-specific vars */
+};
+
+void rioInitWithConnset(rio *r, connection **conns, int numconns) {
+    *r = rioConnsetIO;
+    r->io.connset.conns = zmalloc(sizeof(connection *) * numconns);
+    r->io.connset.state = zmalloc(sizeof(int) * numconns);
+    for (int i = 0; i < numconns; i++) {
+        r->io.connset.conns[i] = conns[i];
+        r->io.connset.state[i] = 0;
+    }
+    r->io.connset.numconns = numconns;
+    r->io.connset.pos = 0;
+    r->io.connset.buf = sdsempty();
+}
+
+/* release the rio stream. */
+void rioFreeConnset(rio *r) {
+    zfree(r->io.connset.conns);
+    zfree(r->io.connset.state);
+    sdsfree(r->io.connset.buf);
 }
