@@ -108,7 +108,7 @@ start_server {tags {"acl external:skip"}} {
     } {*NOPERM*channel*}
 
     test {By default, only default user is able to subscribe to any channel} {
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         $rd AUTH default pwd
         $rd read
         $rd SUBSCRIBE foo
@@ -124,7 +124,7 @@ start_server {tags {"acl external:skip"}} {
     } {*NOPERM*channel*}
 
     test {By default, only default user is able to subscribe to any shard channel} {
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         $rd AUTH default pwd
         $rd read
         $rd SSUBSCRIBE foo
@@ -140,7 +140,7 @@ start_server {tags {"acl external:skip"}} {
     } {*NOPERM*channel*}
 
     test {By default, only default user is able to subscribe to any pattern} {
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         $rd AUTH default pwd
         $rd read
         $rd PSUBSCRIBE bar*
@@ -209,7 +209,7 @@ start_server {tags {"acl external:skip"}} {
     }
 
     test {It's possible to allow subscribing to a subset of channels} {
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         $rd AUTH psuser pspass
         $rd read
         $rd SUBSCRIBE foo:1
@@ -222,7 +222,7 @@ start_server {tags {"acl external:skip"}} {
     } {*NOPERM*channel*}
 
     test {It's possible to allow subscribing to a subset of shard channels} {
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         $rd AUTH psuser pspass
         $rd read
         $rd SSUBSCRIBE foo:1
@@ -235,7 +235,7 @@ start_server {tags {"acl external:skip"}} {
     } {*NOPERM*channel*}
 
     test {It's possible to allow subscribing to a subset of channel patterns} {
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         $rd AUTH psuser pspass
         $rd read
         $rd PSUBSCRIBE foo:1
@@ -248,7 +248,7 @@ start_server {tags {"acl external:skip"}} {
     } {*NOPERM*channel*}
     
     test {Subscribers are killed when revoked of channel permission} {
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         r ACL setuser psuser resetchannels &foo:1
         $rd AUTH psuser pspass
         $rd read
@@ -262,7 +262,7 @@ start_server {tags {"acl external:skip"}} {
     } {0}
 
     test {Subscribers are killed when revoked of channel permission} {
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         r ACL setuser psuser resetchannels &foo:1
         $rd AUTH psuser pspass
         $rd read
@@ -276,7 +276,7 @@ start_server {tags {"acl external:skip"}} {
     } {0}
 
     test {Subscribers are killed when revoked of pattern permission} {
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         r ACL setuser psuser resetchannels &bar:*
         $rd AUTH psuser pspass
         $rd read
@@ -290,7 +290,7 @@ start_server {tags {"acl external:skip"}} {
     } {0}
 
     test {Subscribers are killed when revoked of allchannels permission} {
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         r ACL setuser psuser allchannels
         $rd AUTH psuser pspass
         $rd read
@@ -304,7 +304,7 @@ start_server {tags {"acl external:skip"}} {
     } {0}
 
     test {Subscribers are pardoned if literal permissions are retained and/or gaining allchannels} {
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         r ACL setuser psuser resetchannels &foo:1 &bar:* &orders
         $rd AUTH psuser pspass
         $rd read
@@ -326,7 +326,7 @@ start_server {tags {"acl external:skip"}} {
     test {blocked command gets rejected when reprocessed after permission change} {
         r auth default ""
         r config resetstat
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         r ACL setuser psuser reset on nopass +@all allkeys
         $rd AUTH psuser pspass
         $rd read
@@ -526,7 +526,7 @@ start_server {tags {"acl external:skip"}} {
         }
     }
 
-    # Note that the order of the generated ACL rules is not stable in Redis
+    # Note that the order of the generated ACL rules is not stable in the server
     # so we need to match the different parts and not as a whole string.
     test {ACL GETUSER is able to translate back command permissions} {
         # Subtractive
@@ -612,7 +612,7 @@ start_server {tags {"acl external:skip"}} {
         r ACL SETUSER adv-test -@all +select|0 +select|0 +debug|segfault +debug
         assert_equal "-@all +select|0 +debug" [dict get [r ACL getuser adv-test] commands]
 
-        # Unnecessary categories are retained for potentional future compatibility
+        # Unnecessary categories are retained for potential future compatibility
         r ACL SETUSER adv-test -@all -@dangerous
         assert_equal "-@all -@dangerous" [dict get [r ACL getuser adv-test] commands]
 
@@ -666,16 +666,16 @@ start_server {tags {"acl external:skip"}} {
          for {set j 0} {$j < 10} {incr j} {
              assert_error "*WRONGPASS*" {r AUTH user1 doo}
          }
-         set entry_id_lastest_error [dict get [lindex [r ACL LOG] 0] entry-id]
+         set entry_id_latest_error [dict get [lindex [r ACL LOG] 0] entry-id]
          set timestamp_created_updated [dict get [lindex [r ACL LOG] 0] timestamp-created]
          set timestamp_last_updated_after_update [dict get [lindex [r ACL LOG] 0] timestamp-last-updated]
-         assert {$entry_id_lastest_error eq $entry_id_initial_error}
+         assert {$entry_id_latest_error eq $entry_id_initial_error}
          assert {$timestamp_last_update_original < $timestamp_last_updated_after_update}
          assert {$timestamp_created_original eq $timestamp_created_updated}
          r ACL setuser user2 >doo
          assert_error "*WRONGPASS*" {r AUTH user2 foo}
          set new_error_entry_id [dict get [lindex [r ACL LOG] 0] entry-id]
-         assert {$new_error_entry_id eq $entry_id_lastest_error + 1 }
+         assert {$new_error_entry_id eq $entry_id_latest_error + 1 }
     }
 
     test {ACL LOG shows failed command executions at toplevel} {
@@ -754,7 +754,7 @@ start_server {tags {"acl external:skip"}} {
     }
 
     test {ACL LOG can distinguish the transaction context (2)} {
-        set rd1 [redis_deferring_client]
+        set rd1 [valkey_deferring_client]
         r ACL SETUSER antirez +incr
 
         r AUTH antirez foo
@@ -802,6 +802,16 @@ start_server {tags {"acl external:skip"}} {
         assert {[dict get $entry username] eq {antirez}}
     }
 
+    test {ACLLOG - zero max length is correctly handled} {
+        r ACL LOG RESET
+        r CONFIG SET acllog-max-len 0
+        for {set j 0} {$j < 10} {incr j} {
+            catch {r SET obj:$j 123}
+        }
+        r AUTH default ""
+        assert {[llength [r ACL LOG]] == 0}
+    }
+
     test {ACL LOG entries are limited to a maximum amount} {
         r ACL LOG RESET
         r CONFIG SET acllog-max-len 5
@@ -813,9 +823,14 @@ start_server {tags {"acl external:skip"}} {
         assert {[llength [r ACL LOG]] == 5}
     }
 
+    test {ACL LOG entries are still present on update of max len config} {
+        r CONFIG SET acllog-max-len 0
+        assert {[llength [r ACL LOG]] == 5}
+    }
+
     test {When default user is off, new connections are not authenticated} {
         r ACL setuser default off
-        catch {set rd1 [redis_deferring_client]} e
+        catch {set rd1 [valkey_deferring_client]} e
         r ACL setuser default on
         set e
     } {*NOAUTH*}
@@ -890,7 +905,7 @@ start_server {tags {"acl external:skip"}} {
     test {ACL load non-existing configured ACL file} {
        catch {r ACL load} err
        set err
-    } {*Redis instance is not configured to use an ACL file*}
+    } {*instance is not configured to use an ACL file*}
 
     # If there is an AUTH failure the metric increases
     test {ACL-Metrics user AUTH failure} {
@@ -1005,16 +1020,76 @@ start_server [list overrides [list "dir" $server_path "acl-pubsub-default" "allc
         set e
     } {*NOPERM*set*}
 
+    test {ACL LOAD only disconnects affected clients} {
+        reconnect
+        r ACL SETUSER doug on nopass resetchannels &test* +@all ~*
+
+        set rd1 [valkey_deferring_client]
+        set rd2 [valkey_deferring_client]
+
+        $rd1 AUTH alice alice
+        $rd1 read
+        $rd1 SUBSCRIBE test1
+        $rd1 read
+
+        $rd2 AUTH doug doug
+        $rd2 read
+        $rd2 SUBSCRIBE test1
+        $rd2 read
+
+        r ACL LOAD
+        r PUBLISH test1 test-message
+
+        # Permissions for 'alice' haven't changed, so they should still be connected
+        assert_match {*test-message*} [$rd1 read]
+
+        # 'doug' no longer has access to "test1" channel, so they should get disconnected
+        catch {$rd2 read} e
+        assert_match {*I/O error*} $e
+
+        $rd1 close
+        $rd2 close
+    }
+
+    test {ACL LOAD disconnects clients of deleted users} {
+        reconnect
+        r ACL SETUSER mortimer on >mortimer ~* &* +@all
+
+        set rd1 [valkey_deferring_client]
+        set rd2 [valkey_deferring_client]
+
+        $rd1 AUTH alice alice
+        $rd1 read
+        $rd1 SUBSCRIBE test
+        $rd1 read
+
+        $rd2 AUTH mortimer mortimer
+        $rd2 read
+        $rd2 SUBSCRIBE test
+        $rd2 read
+
+        r ACL LOAD
+        r PUBLISH test test-message
+
+        # Permissions for 'alice' haven't changed, so they should still be connected
+        assert_match {*test-message*} [$rd1 read]
+
+        # 'mortimer' has been deleted, so their client should get disconnected
+        catch {$rd2 read} e
+        assert_match {*I/O error*} $e
+
+        $rd1 close
+        $rd2 close
+    }
+
     test {ACL load and save} {
         r ACL setuser eve +get allkeys >eve on
         r ACL save
 
-        # ACL load will free user and kill clients
         r ACL load
-        catch {r ACL LIST} e
-        assert_match {*I/O error*} $e
 
-        reconnect
+        # Clients should not be disconnected since permissions haven't changed
+
         r AUTH alice alice
         r SET key value
         r AUTH eve eve
@@ -1028,12 +1103,10 @@ start_server [list overrides [list "dir" $server_path "acl-pubsub-default" "allc
         r ACL setuser harry on nopass resetchannels &test +@all ~*
         r ACL save
 
-        # ACL load will free user and kill clients
         r ACL load
-        catch {r ACL LIST} e
-        assert_match {*I/O error*} $e
 
-        reconnect
+        # Clients should not be disconnected since permissions haven't changed
+
         r AUTH harry anything
         r publish test bar
         catch {r publish test1 bar} e
